/*
 * crt2.c
 *
 * Source code for the startup proceedures used by all programs. This code
 * is compiled to make crt1.o, which should be located in the library path.
 *
 * This code is part of the Mingw32 package.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *  Pedro A. Aranda gave me a clue about how to get the standard file
 *    handles, which he in turn was told by Jacob Navia.
 *
 *  Hacked by Mumit Khan <khan@xraylith.wisc.edu> to adapt to U/WIN.
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRENTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warrenties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#undef extern

#include <stdlib.h>
#include <float.h>
#include <process.h>
#include <windows.h>

#define ISSPACE(a)	(a == ' ' || a == '\t')

extern int PASCAL WinMain (HANDLE hInst, HANDLE hPrevInst, LPSTR szCmdLine,
			   int nShow);
/* force linking in the user supplied (or library default) WinMain */
int (*_crt_dummy_linkvar) (HANDLE, HANDLE, LPSTR, int) = WinMain;

#undef WinMain
extern int PASCAL WinMain (HANDLE hInst, HANDLE hPrevInst, LPSTR szCmdLine,
			   int nShow);
/*
 * Access to a standard 'main'-like argument count and list. Also included
 * is a table of environment variables.
 */
int __argc = 0;
char **__argv = 0;
char **_environ = 0;
int errno = 0;

static void
_crt_to_unix (char *arg)
{
  /* convert argv[0] to Unix specs */
  char *cp = arg;
  if (cp[1] == ':')
    {
       cp[1] = cp[0];
       cp[0] = '/';
       cp += 2;
    }
  do
    {
      if(*cp=='\\')
	*cp = '/';
    } 
  while(*cp++);
}

static int
_crt_dummy ()
{
  char *szCmd;
  STARTUPINFO startinfo;
  int nRet;

  /* Get the command line passed to the process. */
  szCmd = GetCommandLineA ();
  GetStartupInfoA (&startinfo);

  (void) __getmainargs (&__argc, &__argv, &_environ, 0, &startinfo);

  /* convert argv[0] to be Unix like */
  _crt_to_unix (__argv[0]);

  /* Strip off the name of the application and any leading
   * whitespace. */
  if (szCmd)
    {
      while (ISSPACE (*szCmd))
	{
	  szCmd++;
	}

      /* On my system I always get the app name enclosed
       * in quotes... */
      if (*szCmd == '\"')
	{
	  do
	    {
	      szCmd++;
	    }
	  while (*szCmd != '\"' && *szCmd != '\0');

	  if (*szCmd == '\"')
	    {
	      szCmd++;
	    }
	}
      else
	{
	  /* If no quotes then assume first token is program
	   * name. */
	  while (!ISSPACE (*szCmd) && *szCmd != '\0')
	    {
	      szCmd++;
	    }
	}

      while (ISSPACE (*szCmd))
	{
	  szCmd++;
	}
    }

  nRet = WinMain (GetModuleHandle (NULL), NULL, szCmd,
		  (startinfo.dwFlags & STARTF_USESHOWWINDOW) ?
		  startinfo.wShowWindow : SW_SHOWDEFAULT);

  return nRet;
}

/*
 * The function mainCRTStartup is the entry point for all console programs.
 */
int
mainCRTStartup ()
{
  int nRet;

  /*
   * I have been told that this is the correct thing to do. You
   * have to uncomment the prototype of SetUnhandledExceptionFilter
   * in the GNU Win32 API headers for this to work. The type it
   * expects is a pointer to a function of the same type as
   * UnhandledExceptionFilter, which is prototyped just above
   * (see Functions.h).
   */
  SetUnhandledExceptionFilter (NULL);

  /*
   * Initialize floating point unit.
   */
  _fpreset ();			/* Supplied by the runtime library. */

  /*
   * Call the main function. If the user does not supply one
   * the one in the 'libmingw32.a' library will be linked in, and
   * that one calls WinMain. See main.c in the 'lib' dir
   * for more details.
   */
  nRet = _crt_dummy ();

  /*
   * Perform exit processing for the C library. This means
   * flushing output and calling 'atexit' registered functions.
   */
  _cexit ();

  ExitProcess (nRet);
}

/*
 * For now the GUI startup function is the same as the console one.
 * This simply gets rid of the annoying warning about not being able
 * to find WinMainCRTStartup when linking GUI applications.
 */
int
WinMainCRTStartup ()
{
  return mainCRTStartup ();
}

